@@ -1152,6 +1152,24 @@ def test_request_param(url):
     assert_reset()
 
 
+def test_request_param_with_multiple_values_for_the_same_key():
+    @responses.activate
+    def run():
+        url = "http://example.com"
+        params = {"key1": ["one", "two"], "key2": "three"}
+        responses.add(
+            method=responses.GET,
+            url=url,
+            body="test",
+        )
+        resp = requests.get(url, params=params)
+        assert_response(resp, "test")
+        assert resp.request.params == params
+
+    run()
+    assert_reset()
+
+
 @pytest.mark.parametrize(
     "url", ("http://example.com", "http://example.com?hello=world")
 )
@@ -1228,3 +1246,56 @@ def test_request_matches_post_params():
 
     run()
     assert_reset()
+
+
+def test_request_matches_empty_body():
+    @responses.activate
+    def run():
+        responses.add(
+            method=responses.POST,
+            url="http://example.com/",
+            body="one",
+            match=[responses.json_params_matcher(None)],
+        )
+
+        responses.add(
+            method=responses.POST,
+            url="http://example.com/",
+            body="two",
+            match=[responses.urlencoded_params_matcher(None)],
+        )
+
+        resp = requests.request("POST", "http://example.com/")
+        assert_response(resp, "one")
+
+        resp = requests.request(
+            "POST",
+            "http://example.com/",
+            headers={"Content-Type": "x-www-form-urlencoded"},
+        )
+        assert_response(resp, "two")
+
+    run()
+    assert_reset()
+
+
+def test_fail_request_error():
+    @responses.activate
+    def run():
+        responses.add("POST", "http://example1.com")
+        responses.add("GET", "http://example.com")
+        responses.add(
+            "POST",
+            "http://example.com",
+            match=[responses.urlencoded_params_matcher({"foo": "bar"})],
+        )
+
+        with pytest.raises(ConnectionError) as excinfo:
+            requests.post("http://example.com", data={"id": "bad"})
+        msg = str(excinfo.value)
+        assert "- POST http://example1.com/ URL does not match" in msg
+        assert "- GET http://example.com/ Method does not match" in msg
+        assert "- POST http://example.com/ Parameters do not match" in msg
+
+    run()
+    assert_reset()
